@@ -2,9 +2,41 @@ import json
 import boto3
 import traceback
 import threading
+from queue import Queue
+from threading import Thread
 import os
 from .coco_convert import get_jsons_dict
 
+class AppWorkerCoco(Thread):
+
+    def __init__(self, queue):
+        Thread.__init__(self)
+        self.queue = queue
+
+    def run(self):
+        while True:
+            image_name,image_data,args,logger = self.queue.get()
+            try:
+                logger.info('runing {}'.format(image_name))
+                upload_coco_annotation(image_name,image_data,args,logger)
+            finally:
+                self.queue.task_done()
+
+class AppWorkerAo(Thread):
+
+    def __init__(self, queue):
+        Thread.__init__(self)
+        self.queue = queue
+
+    def run(self):
+        while True:
+            file_path,args,logger = self.queue.get()
+            try:
+                logger.info('runing {}'.format(file_path))
+                upload_ao_annotations(file_path,args,logger)
+            finally:
+                self.queue.task_done()
+
 
 def upload_coco_annotation(image_name,image_data,args,logger):
     try:
@@ -34,19 +66,33 @@ def upload_ao_annotations(file_path,args,logger):
         logger.error(traceback.format_exc())
 
 def start(args,logger):
+    queue = Queue()    
     if(args.ao_jsons):
+        for x in range(8):
+            worker = AppWorkerAo(queue)
+            worker.daemon = True
+            worker.start()
         pwd = args.ao_jsons
         files = os.listdir(pwd)
         if pwd[-1] != '/':
             pwd += '/'
         for f in files:
+            if not '___objects.json' in f:
+                continue
             file_path = pwd + f
-            t = threading.Thread(target=upload_ao_annotations, args=(file_path,args,logger)).start()
+            logger.info('Queueing {}'.format(file_path))
+            queue.put((file_path,args,logger))            
+        queue.join()
 
     if(args.coco_json):
+        for x in range(8):
+            worker = AppWorkerCoco(queue)
+            worker.daemon = True
+            worker.start()
         coco_json_path = args.coco_json
         ao_jsons = get_jsons_dict(coco_json_path)
         for image_name in ao_jsons:
-            t = threading.Thread(target = upload_coco_annotation, args=(image_name,ao_jsons[image_name],args,logger)).start()
-
+            logger.info('Queueing {}'.format(image_name))
+            queue.put((image_name,ao_jsons[image_name],args,logger))
+        queue.join()
 
